#include <iostream>
#include <ostream>

#include <visionaray/math/math.h>

int main()
{
    using namespace visionaray;

    // 4x4 matrices
    {
        mat4 m1 = mat4::identity();

        mat4 m2(
            1,0,0,0, // First column(! not row)
            0,1,0,0, 
            0,0,1,0,
            2,2,2,1
            );

        // Output.
        std::cout << "m2: " << m2 << '\n';

        // Matrix / matrix product.
        std::cout << "m1 * m2: " << (m1 * m2) << '\n';

        // Matrix / vector product.
        vec4 v4(0, 0, 0, 1);
        std::cout << "m2 * v4: " << (m2 * v4) << '\n';

        // Matrix / vector product with vec3 (promote vec3 to vec4).
        // Mind the floating-point constant: must be vec3::value_type
        vec3 v3(1, 2, 3);
        std::cout << "m2 * vec4(v3, 1.0f): " << (m2 * vec4(v3, 1.0f)) << '\n';

        // Comparisons.
        std::cout << "m1 == identity: " << (m1 == mat4(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1)) << '\n';

        // Transpose.
        std::cout << "transpose(m2): " << transpose(m2) << '\n';

        // Inverse.
        std::cout << "inverse(m2): " << inverse(m2) << '\n';

        // Concatenation.
        std::cout << "inverse(transpose(m1 * m2)): " << inverse(transpose(m1 * m2)) << '\n';

        // See full function list:
        // {PATH_TO}/visionaray/include/visionaray/math/detail/matrix.inl
        // {PATH_TO}/visionaray/include/visionaray/math/detail/matrix2.inl
        // {PATH_TO}/visionaray/include/visionaray/math/detail/matrix3.inl
        // {PATH_TO}/visionaray/include/visionaray/math/detail/matrix4.inl
    }


    // vectors
    {
        vec4 v4(1,2,3,4);

        // Output.
        std::cout << v4 << '\n';

        // Member access.
        std::cout << "v4.x v4.y v4.z v4.w" << v4.x << ' ' << v4.y << ' ' << v4.z << v4.w << '\n';
        v4.x = 2; v4.y = 3; v4.z = 4; v4.w = 5;
        std::cout << "v4.x v4.y v4.z v4.w" << v4.x << ' ' << v4.y << ' ' << v4.z << v4.w << '\n';

        // Conversion
        vec3 v3 = v4.xyz();
        vec2 v2 = v4.xy();
        std::cout << "v2: " << v2 << '\n';
        std::cout << "v3: " << v3 << '\n';

        // Basic arithmetic (similar for vec2 and vec4)!
        std::cout << "v3 + v3: " << (v3 + v3) << '\n';
        std::cout << "v3 - v3: " << (v3 - v3) << '\n';
        std::cout << "v3 * v3: " << (v3 * v3) << '\n';
        std::cout << "v3 / v3: " << (v3 / v3) << '\n';

        v3 += vec3(1,1,1);
        v3 -= vec3(1,1,1);
        v3 *= vec3(1,1,1);
        v3 /= vec3(1,1,1);
        std::cout << "v3: " << v3 << '\n';

        // Vector normalization (not modifying!)
        std::cout << "normalize(v3): " << normalize(v3) << '\n';
        std::cout << "length(normalize(v3)): " << length(normalize(v3)) << '\n';

        // Dot and cross product
        std::cout << "dot(v3, vec3(5,6,7)): " << dot(v3, vec3(5,6,7)) << '\n';
        std::cout << "cross(v3, vec3(5,6,7)): " << cross(v3, vec3(5,6,7)) << '\n';


        // Integer vectors
        vec3i vi(1,1,1);
        std::cout << "vi: " << vi << '\n';

        // See full function list:
        // {PATH_TO}/visionaray/include/visionaray/math/detail/vector.inl
        // {PATH_TO}/visionaray/include/visionaray/math/detail/vector2.inl
        // {PATH_TO}/visionaray/include/visionaray/math/detail/vector3.inl
        // {PATH_TO}/visionaray/include/visionaray/math/detail/vector4.inl
    }
}
